package com.ghostsq.commander.sftp;

import android.util.Log;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.KeyPair;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UIKeyboardInteractive;
import com.jcraft.jsch.UserInfo;

import org.junit.Test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 * Example local unit test, which will execute on the development machine (host).
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
public class KeysTest {
    final static String TAG = "JSch.test";
    private JSch jsch = new JSch();

/*
 correct format can be created by:
 openssl rsa -in ../StudioProjects/GhostCommander/jsch/test/id_rsa -out des3_rsa -des3
*/
//    @Test
    public void changePassphrase() throws JSchException, IOException {
        String key_name = "open_ssh_default.pem";
        File in_file = new File( "test/" + key_name );
        assertTrue( isReencryptable( in_file ) );
        KeyPair keyPair = KeyPair.load( jsch, in_file.getAbsolutePath() );
        assertNotNull( keyPair );
        assertTrue( keyPair.getKeyType() > 0 );

        final String old_passphrase = "testing";
        assertTrue( keyPair.decrypt( old_passphrase ) );
        File out_file = new File( "test/out_" + key_name );
        final String new_passphrase = "this is a test!!!";
        out_file.delete();
        keyPair.writePrivateKey( out_file.getAbsolutePath(), new_passphrase.getBytes( "UTF-8" ) );
        assertTrue( out_file.exists() );
        out_file.setReadable( false, false );
        out_file.setReadable( true, true );
        keyPair = KeyPair.load( jsch, out_file.getAbsolutePath() );
        assertNotNull( keyPair );
        assertTrue( keyPair.getKeyType() > 0 );
        assertTrue( keyPair.isEncrypted() );
        assertTrue( keyPair.decrypt( new_passphrase ) );
    }

//    @Test
    public void generatePair() throws JSchException, IOException {
        File out_file = new File( "test/id_rsa_out" );
        File out_pub_file = new File( "test/id_rsa_out.pub" );
        out_file.delete();
        out_pub_file.delete();
        KeyPair keyPair = KeyPair.genKeyPair( jsch, KeyPair.RSA );

        final String new_passphrase = "This is a new passphrase. Enjoy!";
        keyPair.writePrivateKey( out_file.getAbsolutePath(), new_passphrase.getBytes( "UTF-8" ) );
        keyPair.writePublicKey( out_pub_file.getAbsolutePath(), "Pub file comment" );
        assertTrue( out_file.exists() );
        assertTrue( out_pub_file.exists() );
        out_file.setReadable( false, false );
        out_file.setReadable( true, true );
        keyPair = KeyPair.load( jsch, out_file.getAbsolutePath() );
        assertNotNull( keyPair );
        assertTrue( keyPair.isEncrypted() );
        assertTrue( keyPair.decrypt( new_passphrase ) );
    }

    public final boolean isReencryptable( File file ) {
        try( BufferedReader br = new BufferedReader(new FileReader( file.getAbsolutePath() ) ) )
        {
            String s;
            while( ( s = br.readLine() ) != null ) {
                if( s.length() == 0 ) return false;
                if( s.indexOf( "OPENSSH" )> 0 ) return false;
                if( s.indexOf( "DEK-Info: DES-EDE3-CBC" ) == 0 ) return true;
            }
        }
        catch( IOException e ) {
            e.printStackTrace();
        }
        return false;
    }


    @Test
    public void connect() {
        try {
            JSch jsch = new JSch();

            try {
                File f_kh = new File( "test", "known_hosts" );
                if( !f_kh.exists() )
                    f_kh.createNewFile();
                jsch.setKnownHosts( f_kh.getAbsolutePath() );
            } catch( Exception e ) {
                Log.e( TAG, "", e );
            }


            Session session = jsch.getSession( "demo", "test.rebex.net", 22 );
//            Session session = jsch.getSession( "username", "localhost", 22 );

            UserInfo ui = new MyUserInfo();
            session.setUserInfo(ui);
            session.connect();

            Channel channel = session.openChannel("sftp");
            channel.connect();
            ChannelSftp c = (ChannelSftp)channel;

            java.util.Vector<ChannelSftp.LsEntry> vv = c.ls( "/" );
            for( int i = 0; i < vv.size(); i++ ) {
                ChannelSftp.LsEntry entry = vv.elementAt( i );
                System.out.println( entry.getLongname() );
            }
            c.get( "/readme.txt", "" );
            assertTrue( true );

        } catch( Exception e ) {
            fail(e.getMessage());
        }
    }

    class MyUserInfo implements UserInfo, UIKeyboardInteractive {
        @Override
        public String getPassphrase() {
            System.out.println( "Wants Passphrase" );
            return null;
        }
        @Override
        public String getPassword() {
            System.out.println( "Wants Password" );
            return "password";
        }
        @Override
        public boolean promptPassword( String message ) {
            System.out.println( "prompts Password: " + message );
            return true;
        }
        @Override
        public boolean promptPassphrase( String message ) {
            System.out.println( "prompts Passphrase: " + message );
            return true;
        }
        @Override
        public boolean promptYesNo( String message ) {
            System.out.println( "prompts Y/N: " + message );
            return true;
        }
        @Override
        public void showMessage( String message ) {
            System.out.println( "Message: " + message );
        }

        @Override
        public String[] promptKeyboardInteractive( String destination, String name, String instruction, String[] prompt, boolean[] echo ) {
            System.out.println( "destination: " + destination );
            System.out.println( "name: " + name );
            System.out.println( "instruction: " + instruction );
            for( String pr : prompt )
                System.out.println( "prompt: " + pr );
            return new String[] { "password" };
        }
    }

}
